@@ -41,7 +41,7 @@ my $maint_commands = Lintian::Data->new('debhelper/maint_commands');
 my $dh_commands_depends = Lintian::Data->new('debhelper/dh_commands', '=');
 my $filename_configs = Lintian::Data->new('debhelper/filename-config-files');
 my $dh_ver_deps= Lintian::Data->new('debhelper/dh_commands-manual', qr/\|\|/o);
-my $dh_addons = Lintian::Data->new('debhelper/dh_addons', '=');
+my $dh_addons = Lintian::Data->new('common/dh_addons', '=');
 my $dh_addons_manual
   = Lintian::Data->new('debhelper/dh_addons-manual', qr/\|\|/o);
 my $compat_level = Lintian::Data->new('debhelper/compat-level',qr/=/);
@@ -64,8 +64,10 @@ sub run {
 
     my ($bdepends_noarch, $bdepends, %build_systems, $uses_autotools_dev_dh);
     my $seen_dh = 0;
+    my $seen_dh_parallel = 0;
     my $seen_python_helper = 0;
     my $seen_python3_helper = 0;
+    my %overrides;
 
     $drules = $droot->child('rules') if $droot;
 
@@ -133,6 +135,7 @@ sub run {
             delete($build_systems{'debhelper'});
             $seen_dh = 1;
             $seencommand = 1;
+            $seen_dh_parallel = 1 if m/--parallel/;
             $needbuilddepends = 1;
             $needtomodifyscripts = 1;
             while (m/\s--with(?:=|\s+)(['"]?)(\S+)\1/go) {
@@ -148,6 +151,10 @@ sub run {
                           "dh ... --with ${orig_addon} (line $.)";
                         $uses_autotools_dev_dh = 1;
                     }
+                    tag 'dh-quilt-addon-but-quilt-source-format',
+                      "dh ... --with ${orig_addon}", "(line $.)"
+                      if $addon eq 'quilt'
+                      and $info->field('format', '') eq '3.0 (quilt)';
                     if (defined $depends) {
                         $missingbdeps_addons{$depends} = $addon;
                     }
@@ -191,6 +198,7 @@ sub run {
             # Can be multiple targets per rule.
             while ($targets =~ /\boverride_(dh_[^\s]+)/g) {
                 my $dhcommand = $1;
+                $overrides{$dhcommand} = $.;
                 # If maintainer is using wildcards, it's unlikely to be a typo.
                 next if ($dhcommand =~ /%/);
                 next if ($dh_commands_depends->known($dhcommand));
@@ -273,7 +281,8 @@ sub run {
     # for brace expansion.
     if ($compat_file and $compat_file->is_open_ok) {
         my $compat_file = $compat_file->file_contents;
-        ($compat) = split(/\n/, $compat_file);
+        ($compat) = my @lines = split(/\n/, $compat_file);
+        tag 'debhelper-compat-file-contains-multiple-levels' if @lines > 1;
         strip($compat);
         if ($compat ne '') {
             my $compat_value = $compat;
@@ -318,6 +327,17 @@ sub run {
         tag 'dh-clean-k-is-deprecated';
     }
 
+    for my $suffix (qw(enable start)) {
+        my $line = $overrides{"dh_systemd_$suffix"};
+        tag 'debian-rules-uses-deprecated-systemd-override',
+          "override_dh_systemd_$suffix", "(line $line)"
+          if $line and $level >= 11;
+    }
+
+    tag 'debian-rules-uses-unnecessary-dh-argument', 'dh ... --parallel',
+      "(line $.)"
+      if $seen_dh_parallel and $level >= 10;
+
     # Check the files in the debian directory for various debhelper-related
     # things.
     for my $file ($droot->children) {
@@ -414,8 +434,9 @@ sub run {
                 while (<$fd>) {
                     next if /^\s*$/;
                     next if (/^\#/ and $level >= 5);
-                    if (m/(?<!\\)\{(?:[^\s\\\},]*?,)*[^\\\}\s,]+,*\}/) {
-                        tag 'brace-expansion-in-debhelper-config-file',$file;
+                    if (m/((?<!\\)\{(?:[^\s\\\}]*?,)+[^\\\}\s,]*,*\})/) {
+                        tag 'brace-expansion-in-debhelper-config-file',$file,
+                          $1,"(line $.)";
                         last;
                     }
                 }
@@ -444,6 +465,16 @@ sub run {
     while (my ($dep, $addon) = each %missingbdeps_addons) {
         tag 'missing-build-dependency-for-dh-addon', "$addon => $dep"
           unless ($bdepends_noarch->implies($dep));
+
+        # As a special case, the python3 addon needs a dependency on
+        # dh-python unless the -dev packages are used.
+        my $pkg = 'dh-python';
+        tag 'missing-build-dependency-for-dh-addon',"$addon => $pkg"
+          if $addon eq 'python3'
+          && $bdepends_noarch->implies($dep)
+          && !$bdepends_noarch->implies(
+            'python3-dev:any | python3-all-dev:any')
+          && !$bdepends_noarch->implies($pkg);
     }
 
     $dh_bd_version = $level if not defined($dh_bd_version);
