@@ -52,9 +52,21 @@ our $known_build_essential
 our $KNOWN_BINARY_FIELDS = Lintian::Data->new('fields/binary-fields');
 our $KNOWN_UDEB_FIELDS = Lintian::Data->new('fields/udeb-fields');
 our $KNOWN_BUILD_PROFILES = Lintian::Data->new('fields/build-profiles');
+our $KNOWN_VCS_BROWSERS
+  = Lintian::Data->new('fields/vcs-browsers', qr/\s*~~\s*/, sub { $_[1]; });
 
 our %KNOWN_ARCHIVE_PARTS = map { $_ => 1 } ('non-free', 'contrib');
 
+my $DERIVATIVE_FIELDS = Lintian::Data->new(
+    'fields/derivative-fields',
+    qr/\s*\~\~\s*/,
+    sub {
+        my ($regexp, $explanation) = split(/\s*\~\~\s*/, $_[1], 2);
+        return {
+            'regexp' => qr/$regexp/,
+            'explanation' => $explanation,
+        };
+    });
 my $KNOWN_PRIOS = Lintian::Data->new('fields/priorities');
 
 our @supported_source_formats = (qr/1\.0/, qr/3\.0\s*\((quilt|native)\)/);
@@ -228,6 +240,7 @@ our $OBSOLETE_PACKAGES
   = Lintian::Data->new('fields/obsolete-packages',qr/\s*=>\s*/);
 our $VIRTUAL_PACKAGES   = Lintian::Data->new('fields/virtual-packages');
 our $SOURCE_FIELDS      = Lintian::Data->new('common/source-fields');
+our $MAIL_TRANSPORT_AGENTS= Lintian::Data->new('fields/mail-transport-agents');
 
 sub run {
     my ($pkg, $type, $info, $proc, $group) = @_;
@@ -741,6 +754,7 @@ sub run {
 
             tag 'alternates-not-allowed', $field
               if ($data =~ /\|/ && !&$is_dep_field($field));
+            check_field($info, $field, $data) if &$is_dep_field($field);
 
             for my $dep (split /\s*,\s*/, $data) {
                 my (@alternatives, @seen_obsolete_packages);
@@ -778,6 +792,11 @@ sub run {
                   if ( &$is_dep_field($field)
                     && $known_obsolete_emacs{$alternatives[0][0]});
 
+                tag 'depends-on-mail-transport-agent-without-alternatives',
+                  $dep
+                  if &$is_dep_field($field)
+                  and $MAIL_TRANSPORT_AGENTS->known($dep);
+
                 for my $part_d (@alternatives) {
                     my ($d_pkg, undef, $d_version, undef, undef, $rest,
                         $part_d_orig)
@@ -1040,6 +1059,7 @@ sub run {
                 #Get data and clean it
                 my $data = $info->field($field);
                 unfold($field, \$data);
+                check_field($info, $field, $data);
                 $depend{$field} = $data;
 
                 for my $dep (split /\s*,\s*/, $data) {
@@ -1104,7 +1124,7 @@ sub run {
                           if ($d_pkg eq 'build-essential');
 
                         #<<< no tidy, tag name too long
-                        tag 'depends-on-build-essential-package-without-using-version',
+                        tag 'build-depends-on-build-essential-package-without-using-version',
                         #>>>
                           "$d_pkg [$field: $part_d_orig]"
                           if ($known_build_essential->known($d_pkg)
@@ -1370,17 +1390,18 @@ sub run {
             if ($vcs eq 'browser') {
                 tag 'vcs-browser-links-to-empty-view', $uri
                   if $uri =~ m%rev=0&sc=0%;
-                next;
+            } else {
+                $seen_vcs{$vcs}++;
             }
-            $seen_vcs{$vcs}++;
-            if ($uri =~ m/\.debian\.org/) {
+            if ($uri =~ m{//([^.]+)\.debian\.org/}) {
                 tag 'vcs-deprecated-in-debian-infrastructure', "vcs-$vcs", $uri
-                  if $vcs ne 'git';
+                  if $1 ne 'salsa';
             } else {
                 tag 'orphaned-package-not-maintained-in-debian-infrastructure',
                   "vcs-$vcs", $uri
                   if $info->field('maintainer', '')
-                  =~ /packages\@qa.debian.org/;
+                  =~ /packages\@qa.debian.org/
+                  and $vcs ne 'browser';
             }
         }
     }
@@ -1392,11 +1413,34 @@ sub run {
       and $is_comaintained
       and not %seen_vcs;
 
+    # Check for missing Vcs-Browser headers
+    if (!defined $info->field('vcs-browser')) {
+        foreach my $regex ($KNOWN_VCS_BROWSERS->all) {
+            my $vcs = $KNOWN_VCS_BROWSERS->value($regex);
+            if ($info->field("vcs-$vcs", '') =~ m/^($regex.*)/xi) {
+                tag 'missing-vcs-browser-field', "vcs-$vcs", $1;
+                last; # Only warn once
+            }
+        }
+    }
+
     #---- Checksums
 
     tag 'no-strong-digests-in-dsc'
       if $type eq 'source' && !$info->field('checksums-sha256');
 
+    #----- Derivative-specific field checks
+
+    if ($type eq 'source') {
+        foreach my $field ($DERIVATIVE_FIELDS->all) {
+            my $val = $info->field($field, '-');
+            my $data = $DERIVATIVE_FIELDS->value($field);
+            tag 'invalid-field-for-derivative',
+              "$field: $val ($data->{'explanation'})"
+              if $val !~ m/$data->{'regexp'}/;
+        }
+    }
+
     #----- Field checks (without checking the value)
 
     for my $field (keys %{$info->field}) {
@@ -1478,6 +1522,28 @@ sub unfold {
     return;
 }
 
+sub check_field {
+    my ($info, $field, $data) = @_;
+    my @seen;
+    $info->relation($field)->visit(sub { push @seen, $_; }, VISIT_PRED_NAME);
+
+    my $has_default_mta = any { $_ eq 'default-mta' } @seen;
+    my $has_mail_transport_agent = any { $_ eq 'mail-transport-agent' } @seen;
+
+    if ($has_default_mta) {
+        tag 'default-mta-dependency-not-listed-first',"$field: $data"
+          if $seen[0] ne 'default-mta';
+        tag 'default-mta-dependency-does-not-specify-mail-transport-agent',
+          "$field: $data"
+          unless $has_mail_transport_agent;
+    } elsif ($has_mail_transport_agent) {
+        tag 'mail-transport-agent-dependency-does-not-specify-default-mta',
+          "$field: $data"
+          unless $has_default_mta;
+    }
+    return;
+}
+
 1;
 
 # Local Variables:
