@@ -36,6 +36,12 @@ my %DJANGO_PACKAGES = (
     '^python3-django-' => 'python3-django',
     '^python2?-django-' => 'python-django',
 );
+
+my %REQUIRED_DEPENDS = (
+    'python2' => 'python-minimal:any | python:any',
+    'python3' => 'python3-minimal:any | python3:any',
+);
+
 my %MISMATCHED_SUBSTVARS = (
     '^python3-.+' => '${python:Depends}',
     '^python2?-.+' => '${python3:Depends}',
@@ -62,8 +68,14 @@ sub _run_source {
         if ($bin =~ /^python2?-(.*)$/) {
             my $suffix = $1;
             next if any { $bin =~ /$_/ } @IGNORE;
-            tag 'python-foo-but-no-python3-foo', $bin
-              unless any { $_ eq "python3-${suffix}" } @package_names;
+            next if any { $_ eq "python3-${suffix}" } @package_names;
+            # Don't trigger if we ship any Python 3 module
+            next if any {
+                $info->binary_relation($_, 'all')
+                  ->implies('${python3:Depends}')
+            }
+            @package_names;
+            tag 'python-foo-but-no-python3-foo', $bin;
         }
     }
 
@@ -93,12 +105,31 @@ sub _run_source {
 sub _run_binary {
     my ($pkg, $info) = @_;
 
+    my $deps = Lintian::Relation->and($info->relation('all'),
+        $info->relation('provides'), $pkg);
     my @entries = $info->changelog ? $info->changelog->data : ();
 
+    # Check for missing dependencies
+    if ($pkg !~ /-dbg$/) {
+        foreach my $file ($info->sorted_index) {
+            if (    $file->is_file
+                and $file
+                =~ m,usr/lib/(?<version>python[23])[\d.]*/(?:site|dist)-packages,
+                and not $deps->implies($REQUIRED_DEPENDS{$+{version}})) {
+                tag 'python-package-missing-depends-on-python';
+                last;
+            }
+        }
+    }
+
     # Python 2 modules
-    if ($pkg =~ /^python2?-/ and none { $pkg =~ /$_$/ } @IGNORE) {
-        tag 'new-package-should-not-package-python2-module'
-          if @entries == 1;
+    if (    $pkg =~ /^python2?-/
+        and none { $pkg =~ /$_$/ } @IGNORE
+        and @entries == 1
+        and $entries[0]->Changes
+        !~ /\bpython ?2(?:\.x)? (?:variant|version)\b/im
+        and index($entries[0]->Changes, $pkg) == -1) {
+        tag 'new-package-should-not-package-python2-module', $pkg;
     }
 
     # Python applications
@@ -116,7 +147,7 @@ sub _run_binary {
     foreach my $regex (keys %DJANGO_PACKAGES) {
         my $basepkg = $DJANGO_PACKAGES{$regex};
         next if $pkg !~ /$regex/;
-        next if $pkg =~ /^python3?-django$/;
+        next if any { $pkg =~ /$_/ } @IGNORE;
         tag 'django-package-does-not-depend-on-django', $basepkg
           if not $info->relation('strong')->implies($basepkg);
     }
